import twmn_mpd.utils as utils
from twmn_mpd.NotificationProvider import NotificationProvider


class PlayStateNotifier(NotificationProvider):
    def state_changed(self, old_state, new_state):
        return old_state.status["state"] != new_state.status["state"]

    def get_notif(self, mpd_state):
        status = mpd_state.status
        header = utils.to_human_state_name(status["state"])
        content = self.get_song_name(mpd_state)

        _title = "<title>" + header + "</title>"
        _content = "<content>" + content + "</content>"

        msg = "<root>" + _title + _content + "</root>"

        return msg



import mpd

from twmn_mpd.MPDState import MPDState


def msg_queue(providers):
    """
    Returns a generator expression of notifications for twmn to display,
    based on a list of NotificationProvider classes.
    """

    mpd_client = get_mpd_client()

    notifiers = [provider() for provider in providers]

    prev_state = MPDState(mpd_client)
    curr_state = MPDState(mpd_client)

    while True:
        try:
            curr_state.update()
            hasChanged = False
            for notifier in notifiers:
                if notifier.state_changed(prev_state, curr_state):
                    yield notifier.get_notif(curr_state)
                    hasChanged = True
            if hasChanged:
                prev_state.update()
        except (ConnectionError, ConnectionRefusedError):
            mpd_client = get_mpd_client()
        except ConnectionRefusedError:
            mpd_client = get_mpd_client()


def get_mpd_client(ip="127.0.0.1", port=6600):
    mpd_client = mpd.MPDClient()
    mpd_client.connect("127.0.0.1", str(port))
    return mpd_client



class MPDState(object):
    def __init__(self, mpd_client):
        self.mpd_client = mpd_client
        self.update()

    def update(self):
        self.song = self.mpd_client.currentsong()
        self.status = self.mpd_client.status()
        self.stats = self.mpd_client.stats()



class NotificationProvider(object):
    def get_state(self, mpd_client):
        """Return the state that the message queue stores."""
        return

    # @abc.abstractmethod
    # def state_changed(self, old_state, new_state):
    #     """Whether the state has changed in a way that merits showing another
    #     notification."""
    #     return

    def get_notif(self, state):
        """Return the notification text to be shown."""

        return

    def get_song_name(self, state):
        song = state.song

        date = song["date"][:4]
        artist = song["artist"]
        title = song["title"]
        content = artist + " - " + title + " " + "({})".format(date)

        print(content)
        return content



from twmn_mpd.NotificationProvider import NotificationProvider


class NowPlayingNotifier(NotificationProvider):
    def state_changed(self, old_state, new_state):
        return old_state.song != new_state.song

    def get_notif(self, mpd_state):
        header = "now playing"
        content = self.get_song_name(mpd_state)

        _title = "<title>" + header + "</title>"
        _content = "<content>" + content + "</content>"

        msg = "<root>" + _title + _content + "</root>"

        return msg



from setuptools import setup

setup(name='twmn_mpd',
      version='0.1.4',
      description='A simple mpd notifier for twmn',
      url='http://github.com/mrkgnao/twmn-mpd',
      author='Soham Chowdhury',
      author_email='chow.soham@gmail.com',
      license='MIT',
      packages=['twmn_mpd', 'twmn_mpd.providers'],
      install_requires=[
          'python-mpd2'
      ],
      scripts=[
          'mpd_twmn_notifier'
      ],
      zip_safe=False)



import socket
from contextlib import closing

import twmn_mpd.MsgQueue as MsgQueue
from twmn_mpd.providers.NowPlayingNotifier import NowPlayingNotifier
from twmn_mpd.providers.PlayStateNotifier import PlayStateNotifier


def create_notifs(sock, notifiers):
    """
    Creates a notification for each message in the queue,
    based on a list of notifiers, by writing to the socket
    provided.
    """

    for msg in MsgQueue.msg_queue(notifiers):
        sock.send(msg.encode())


def start_notifs(ip="127.0.0.1",
                 port=9797,
                 notifiers=[PlayStateNotifier, NowPlayingNotifier]):
    """
    The main entrypoint to twmn_mpd.
    """

    with closing(socket.socket(socket.AF_INET,
                               socket.SOCK_DGRAM)) as twmn_sock:
        twmn_sock.connect((ip, port))
        create_notifs(twmn_sock, notifiers)
